package net.lensfrex.oj.collector;

import net.lensfrex.oj.collector.data.Result;
import net.lensfrex.oj.collector.data.Sample;
import net.lensfrex.oj.collector.utils.IOUtil;
import net.lensfrex.oj.collector.utils.ParamFiller;

import java.io.File;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ResourceBundle;
import java.util.regex.Pattern;

public class Record {
    private static final String MAIN_TEMPLATE = IOUtil.inputStreamToString(Record.class.getResourceAsStream("/template.txt"), StandardCharsets.UTF_8);
    private static final String EXAMPLE_TEMPLATE = IOUtil.inputStreamToString(Record.class.getResourceAsStream("/exampleTemplates.txt"), StandardCharsets.UTF_8);

    public String fillText(Result results) {
        HashMap<String, String> params = new HashMap<>();
        params.put("title", results.getTitle());
        params.put("description", results.getDescription());
        params.put("inputDescription", results.getInputDescription());
        params.put("outputDescription", results.getOutputDescription());
        params.put("timeLimit", results.getTimeLimit().toString());
        params.put("memoryLimit", results.getMemoryLimit().toString());
        params.put("level", results.getDifficulty());
        params.put("id", results.getId());
        params.put("tags", results.getTags().toString());
        params.put("hint", results.getHint());

        StringBuffer exampleStringBuffer = new StringBuffer();
        List<Sample> samples = results.getSamples();
        int sampleSize = samples.size();
        for (int i = 0; i < sampleSize; i++) {
            exampleStringBuffer.append(fillExampleText(samples.get(i), i + 1));
        }

        params.put("examples", exampleStringBuffer.toString());

        return ParamFiller.fill(MAIN_TEMPLATE, params);
    }

    private String fillExampleText(Sample sample, int index) {
        HashMap<String, String> params = new HashMap<>();
        if (index == 1) {
            params.put("index", "");

        } else {
            params.put("index", String.valueOf(index));
        }
        params.put("inputExamples", sample.getInput());
        params.put("outputExamples", sample.getOutput());

        return ParamFiller.fill(EXAMPLE_TEMPLATE, params);
    }

    private static final Pattern FilePattern = Pattern.compile("[\\\\/:*?\"<>|]");

    public static String filenameFilter(String str) {
        return str == null ? null : FilePattern.matcher(str).replaceAll("");
    }

    public void writeToFile(String location, String id, String title, String context) {
        id = filenameFilter(id);
        title = filenameFilter(title);

        File file = new File(String.format("%s/%s - %s.md", location, id, title));
        System.out.println("Will write to: " + file.getPath());
        try {
            IOUtil.writeFile(context.getBytes(StandardCharsets.UTF_8), file);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
